/* 
   File: zddesk.cpp - Implementation of main CWinApp object.

   Implements ZDDesk's main application object and also implements the 
   about dialog box.

   Author: Steven E. Sipe
   
*/

#include "stdafx.h"
#include "zddesk.h"

#include "frame.h"
#include "splash.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CZDDeskApp

BEGIN_MESSAGE_MAP(CZDDeskApp, CWinApp)
   //{{AFX_MSG_MAP(CZDDeskApp)
   //}}AFX_MSG_MAP

   // Standard file based document commands
   ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
   ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CZDDeskApp construction

// Constructor
CZDDeskApp::CZDDeskApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// App object

// The one and only application object
CZDDeskApp theApp;

// Determine if we are running the new shell (i.e. Windows 95 or Windows NT 4.0
// or later)
BOOL CZDDeskApp::HasNewShell()
{
   // Get the Windows Version
   DWORD dwVersion = GetVersion();

   // Windows NT?
   if(dwVersion < 0x80000000) 
   {
       // Yes, Windows NT 4.0 or greater?
       if (LOBYTE(LOWORD(dwVersion)) >= 4) 
         return TRUE; 
   } // Windows 95?
   else if(LOBYTE(LOWORD(dwVersion)) >= 4) 
         return TRUE;

   return FALSE;  // Win16, 32s or NT3.x.
}


// Determine if ZDDesk is already running by using a Mutex object -- returns
// TRUE if already running, FALSE otherwise.  Note that a mutex is created
// with the name of the application
BOOL CZDDeskApp::AlreadyRunning()
{
   BOOL bFound = FALSE;

   // Try to create a mutex with the app's name
   HANDLE hMutexOneInstance = ::CreateMutex(NULL,TRUE,_T(AfxGetAppName()));

   // Already there...means that we are already running an instance
   if(::GetLastError() == ERROR_ALREADY_EXISTS)
      bFound = TRUE;

   // Release the mutex
   if(hMutexOneInstance)
      ::ReleaseMutex(hMutexOneInstance);

   return(bFound);
}

/////////////////////////////////////////////////////////////////////////////
// CZDDeskApp initialization

// Performs instance initialization for the application
BOOL CZDDeskApp::InitInstance()
{
   // Command line info -- used by the splash screen
   CCommandLineInfo cmdInfo;
   ParseCommandLine(cmdInfo);

   // Only allow one instance of our application to execute
   if(AlreadyRunning())
   {
      AfxMessageBox(IDS_ALREADY_RUNNING,MB_OK|MB_ICONWARNING);
      return(FALSE);
   }

   // Only run with the new shell --- we need a system tray
   if(!HasNewShell())
   {
      AfxMessageBox(IDS_WRONG_SHELL,MB_OK|MB_ICONSTOP);
      return(FALSE);
   }

   // Setup the splash screen
   CSplashWnd::EnableSplashScreen(cmdInfo.m_bShowSplash);

   // Enable 3D controls
   Enable3dControlsStatic();

   // Construct the main window and load its definition
   CZDDeskFrame *pFrame = new CZDDeskFrame();
   if(!pFrame->LoadFrame(IDR_MAINFRAME,WS_OVERLAPPEDWINDOW))
      return(FALSE);

   // Assign it as the main window, then hide it.  We never display it, 
   // we just use it to receive notification messages from the tray
   m_pMainWnd = pFrame;
   m_pMainWnd->ShowWindow(SW_HIDE);
   m_pMainWnd->UpdateWindow();

   // Load the common control DLL
   ::InitCommonControls();
   
   return(TRUE);
}

// Call winhelp to display topics from our help file
void CZDDeskApp::WinHelp(DWORD dwData, UINT nCmd)
{
   // Calls ZDDesk's help file (zddesk.hlp) -- (we don't really 
   // need to override this, but example sake it is here.  The
   // default help file name loaded by MFC would be zddesk.hlp 
   // anyway
   ::WinHelp(AfxGetMainWnd()->m_hWnd,"zddesk.hlp",nCmd,dwData);
}

/////////////////////////////////////////////////////////////////////////////
// CZDDeskApp commands

// Grab the messages that are needed by the splash screen
BOOL CZDDeskApp::PreTranslateMessage(MSG* pMsg)
{
   // CG: The following lines were added by the Splash Screen component.
   if (CSplashWnd::PreTranslateAppMessage(pMsg))
      return TRUE;

   // Call the base class
   return CWinApp::PreTranslateMessage(pMsg);
}

/////////////////////////////////////////////////////////////////////////////
// CAbout dialog

// Constructor
CAbout::CAbout(CWnd* pParent /*=NULL*/)
   : CDialog(CAbout::IDD, pParent)
{
   //{{AFX_DATA_INIT(CAbout)
      // NOTE: the ClassWizard will add member initialization here
   //}}AFX_DATA_INIT
}

// Bind data members for dialog controls
void CAbout::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CAbout)
      // NOTE: the ClassWizard will add DDX and DDV calls here
   //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAbout, CDialog)
   //{{AFX_MSG_MAP(CAbout)
   ON_WM_CTLCOLOR()
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAbout message handlers

// Handles control colors -- overridden to change the background color of the
// dialog to light gray so that the ZDDesk logo will look correct no matter
// what background color we have set
HBRUSH CAbout::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
   // We only care about the dialog's color
   if(nCtlColor == CTLCOLOR_DLG)
   {
      // Set the background color to light gray
      static HBRUSH hbr = ::CreateSolidBrush(RGB(192,192,192));
      return(hbr);
   }  // Call the default handling for the rest
   else return CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
}
